@@ -1,10 +1,9 @@
-(*  Orpie -- a stack-based RPN calculator for the console
- *  Copyright (C) 2003-2004  Paul Pelzl
+(*  Orpie -- a fullscreen RPN calculator for the console
+ *  Copyright (C) 2003-2004, 2005, 2006-2007 Paul Pelzl
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  it under the terms of the GNU General Public License, Version 2,
+ *  as published by the Free Software Foundation.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -59,7 +58,7 @@
 open Rpc_calc;;
 open Rpc_stack;;
 
-let calc = new Rpc_calc.rpc_calc in
+let calc = new Rpc_calc.rpc_calc false in
 
 (* load data into the calculator using its string representation *)
 let load_data (data_str : string) =
