@@ -1,3 +1,6 @@
+/* ocamlgsl - OCaml interface to GSL                        */
+/* Copyright (©) 2002-2005 - Olivier Andrieu                */
+/* distributed under the terms of the GPL version 2         */
 
 #include <gsl/gsl_complex.h>
 
@@ -23,8 +26,3 @@ copy_complex(gsl_complex_float * c)
 
 #define _CONVERT_COMPLEX(v) GSL_SET_COMPLEX(&z_##v,Double_field(v, 0), Double_field(v,1))
 #define _CONVERT_COMPLEX2(v1,v2) _CONVERT_COMPLEX(v1); _CONVERT_COMPLEX(v2)
-
-#if __GNUC__ >= 3
-#define DECLARE_COMPLEX(v) _DECLARE_COMPLEX(v); _CONVERT_COMPLEX(v)
-#define DECLARE_COMPLEX2(v1,v2) DECLARE_COMPLEX(v1); DECLARE_COMPLEX(v2)
-#endif /* __GNUC__ */
