@@ -1,10 +1,9 @@
-(*  Orpie -- a stack-based RPN calculator for the console
- *  Copyright (C) 2003-2004  Paul Pelzl
+(*  Orpie -- a fullscreen RPN calculator for the console
+ *  Copyright (C) 2003-2004, 2005, 2006-2007 Paul Pelzl
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  it under the terms of the GNU General Public License, Version 2,
+ *  as published by the Free Software Foundation.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -19,6 +18,10 @@
  *  <pelzlpj@eecs.umich.edu>.
  *)
 
+(* interface_draw.ml
+ * This file has all code concerned with rendering the stack, help panel,
+ * and data entry line. *)
+
 
 open Curses;;
 open Printf;;
@@ -95,44 +98,39 @@ let draw_stack (iface : interface_state_t) =
       assert (wmove iface.scr.stack_win 
       (iface.scr.sw_lines + iface.stack_bottom_row - 1 - line) 0);
       wclrtoeol iface.scr.stack_win;
-      begin
-         if line = iface.stack_selection && 
-         iface.interface_mode = BrowsingMode then
-            wattron iface.scr.stack_win WA.reverse
-         else
-            ()
+      if line = iface.stack_selection && 
+      iface.interface_mode = BrowsingMode then
+         wattron iface.scr.stack_win WA.reverse
+      else
+         ();
+      if len > iface.scr.sw_cols - 7 then begin
+         (* need to truncate the string *)
+         let line_string =
+            if line = iface.stack_selection && 
+            iface.interface_mode = BrowsingMode then
+               let sub_s = 
+                  if iface.horiz_scroll < len - iface.scr.sw_cols + 7 then
+                     String.sub s iface.horiz_scroll (iface.scr.sw_cols - 7)
+                  else
+                     String.sub s (len - iface.scr.sw_cols + 7) 
+                     (iface.scr.sw_cols - 7)
+               in
+               print_numbered_line line sub_s
+            else
+               let sub_s = String.sub s 0 (iface.scr.sw_cols - 10) in
+               print_numbered_line line (sub_s ^ "...")
+         in
+         assert (waddstr iface.scr.stack_win line_string)
+      end else begin
+         let spacer = String.make (iface.scr.sw_cols - 7 - len) ' ' in
+         let line_string = print_numbered_line line (spacer ^ s) in 
+         assert (waddstr iface.scr.stack_win line_string)
       end;
-      begin
-         if len > iface.scr.sw_cols - 7 then
-            (* need to truncate the string *)
-            let line_string =
-               if line = iface.stack_selection && 
-               iface.interface_mode = BrowsingMode then
-                  let sub_s = 
-                     if iface.horiz_scroll < len - iface.scr.sw_cols + 7 then
-                        String.sub s iface.horiz_scroll (iface.scr.sw_cols - 7)
-                     else
-                        String.sub s (len - iface.scr.sw_cols + 7) 
-                        (iface.scr.sw_cols - 7)
-                  in
-                  print_numbered_line line sub_s
-               else
-                  let sub_s = String.sub s 0 (iface.scr.sw_cols - 10) in
-                  print_numbered_line line (sub_s ^ "...")
-            in
-            assert (waddstr iface.scr.stack_win line_string)
-         else
-            let spacer = String.make (iface.scr.sw_cols - 7 - len) ' ' in
-            let line_string = print_numbered_line line (spacer ^ s) in 
-            assert (waddstr iface.scr.stack_win line_string)
-      end;
-      begin
-         if line = iface.stack_selection && 
-         iface.interface_mode = BrowsingMode then
-            wattroff iface.scr.stack_win WA.reverse
-         else
-            ()
-      end
+      if line = iface.stack_selection && 
+      iface.interface_mode = BrowsingMode then
+         wattroff iface.scr.stack_win WA.reverse
+      else
+         ();
    done;
    assert (wnoutrefresh iface.scr.stack_win);
    assert (wmove iface.scr.entry_win (iface.scr.ew_lines - 1) (iface.scr.ew_cols - 1))
@@ -292,25 +290,36 @@ let draw_entry (iface : interface_state_t) =
          "@ " ^ iface.variable_entry_buffer
    in
    begin match iface.interface_mode with
-   |StandardEntryMode ->
+   |StandardEditMode ->
       draw_entry_string data_string 0
    |IntEditMode ->
       draw_entry_string data_string 0
-   |AbbrevEntryMode ->
+   |AbbrevEditMode ->
+      let first_abbrev_match =
+         if iface.matched_abbrev_list = [] then ""
+         else List.hd iface.matched_abbrev_list
+      in
       let highlight_len = String.length iface.abbrev_entry_buffer in
       if highlight_len = 0 then
-         draw_entry_string "<enter command abbreviation>" 0
+         begin match iface.abbrev_or_const with
+         |IsAbbrev -> draw_entry_string "<enter command abbreviation>" 0
+         |IsConst  -> draw_entry_string "<enter constant symbol>" 0
+         end
       else
-         let is_function =
-            match (Rcfile.translate_abbrev iface.matched_abbrev_entry) with
-            |Function ff -> true
-            |_ -> false
-         in
-         if is_function then
-            draw_entry_string (iface.matched_abbrev_entry ^ 
-            "( )") highlight_len
-         else
-            draw_entry_string iface.matched_abbrev_entry highlight_len
+         begin match iface.abbrev_or_const with
+         |IsAbbrev ->
+            let is_function =
+               match (Rcfile.translate_abbrev first_abbrev_match) with
+               |Function ff -> true
+               |_ -> false
+            in
+            if is_function then
+               draw_entry_string (first_abbrev_match ^ "( )") highlight_len
+            else
+               draw_entry_string first_abbrev_match highlight_len
+         |IsConst ->
+            draw_entry_string first_abbrev_match highlight_len
+         end
    |BrowsingMode ->
       ()
    |VarEditMode ->
@@ -404,6 +413,275 @@ let generate_abbrev_help () =
 
 
 
+(* create the list of constants to display in the abbrev command
+ * help screen *)
+let generate_const_help () =
+   let rec trunc_list lst n =
+      if n = 0 then
+         []
+      else
+         match lst with
+         |[] ->
+            []
+         |head :: tail ->
+            head :: (trunc_list tail (pred n))
+   in
+   let rec make_symbols_string symbols_list symbols_str =
+      match symbols_list with
+      | [] ->
+         symbols_str
+      | head :: tail ->
+         make_symbols_string tail (head ^ "  " ^ symbols_str)
+   in
+   let symbols = make_symbols_string !Rcfile.constant_symbols "" in
+   trunc_list (Utility.wordwrap_nspace symbols 34 2) 5
+
+
+
+(* draw the help page in standard entry mode *)
+let draw_help_standard iface win mvwaddstr_safe try_find =
+   if iface.help_page = 0 then begin
+      wattron win WA.bold;
+      assert (mvwaddstr win 5 0 "Common Operations:");
+      wattroff win WA.bold;
+      mvwaddstr_safe win 6 2  ("enter    : " ^
+      try_find (Edit Enter));
+      mvwaddstr_safe win 7 2  ("drop     : " ^
+      try_find (Command Drop));
+      mvwaddstr_safe win 8 2  ("swap     : " ^
+      try_find (Command Swap));
+      mvwaddstr_safe win 9 2  ("backspace: " ^
+      try_find (Edit Backspace));
+      mvwaddstr_safe win 10 2 ("add      : " ^
+      try_find (Function Add));
+      mvwaddstr_safe win 11 2 ("subtract : " ^
+      try_find (Function Sub));
+      mvwaddstr_safe win 12 2 ("multiply : " ^
+      try_find (Function Mult));
+      mvwaddstr_safe win 13 2 ("divide   : " ^
+      try_find (Function Div));
+      mvwaddstr_safe win 14 2 ("y^x      : " ^
+      try_find (Function Pow));
+      mvwaddstr_safe win 15 2 ("negation : " ^
+      try_find (Function Neg));
+      wattron win WA.bold;
+      mvwaddstr_safe win 16 0 "Miscellaneous:";
+      wattroff win WA.bold;
+      mvwaddstr_safe win 17 2 ("scientific notation     : " ^
+      try_find (Edit SciNotBase));
+      mvwaddstr_safe win 18 2 ("abbreviation entry mode : " ^
+      try_find (Command BeginAbbrev));
+      mvwaddstr_safe win 19 2 ("stack browsing mode     : " ^
+      try_find (Command BeginBrowse));
+      mvwaddstr_safe win 20 2 ("refresh display         : " ^
+      try_find (Command Refresh));
+      mvwaddstr_safe win 21 2 ("quit                    : " ^
+      try_find (Command Quit));
+      assert (wnoutrefresh win)
+   end else begin
+      let adjust_len s len =
+         if String.length s < len then
+            s ^ (String.make (len - (String.length s)) ' ')
+         else
+            Str.string_before s len
+      in
+      let make_string colon_pos key_string abbr =
+         (adjust_len key_string colon_pos) ^ ": " ^ abbr
+      in
+      wattron win WA.bold;
+      mvwaddstr_safe win 5 0 "Autobindings:";
+      wattroff win WA.bold;
+      if Array.length !Rcfile.autobind_keys <= 0 then
+         mvwaddstr_safe win 6 2 "(none)"
+      else
+         for i = 0 to pred (min (iface.scr.hw_lines - 6) (Array.length
+         !Rcfile.autobind_keys)) do
+            let (key, key_string, bound_f, age) = !Rcfile.autobind_keys.(i) in
+            let abbr = match bound_f with
+               |None    -> "(none)"
+               |Some op -> Rcfile.abbrev_of_operation op
+            in
+            mvwaddstr_safe win (i + 6) 2 (make_string 12 key_string abbr)
+         done;
+      assert (wnoutrefresh win)
+   end
+
+
+(* draw help page in integer editing mode *)
+let draw_help_intedit iface win mvwaddstr_safe try_find =
+   wattron win WA.bold;
+   mvwaddstr_safe win 5 0 "Integer Editing Operations:";
+   wattroff win WA.bold;
+   mvwaddstr_safe win 6 2 ("enter    : " ^
+   try_find (Edit Enter));
+   mvwaddstr_safe win 7 2 ("set base : " ^
+   try_find (Edit SciNotBase));
+   mvwaddstr_safe win 8 2 ("cancel   : " ^
+   try_find (IntEdit IntEditExit));
+   assert (wnoutrefresh win)
+
+
+(* draw help page in abbrev/constant entry mode *)
+let draw_help_abbrev iface win mvwaddstr_safe try_find =
+   if String.length iface.abbrev_entry_buffer = 0 then begin
+      let abbr_strings = generate_abbrev_help () in
+      let const_strings = generate_const_help () in
+      let rec print_help_lines lines v_pos =
+         begin match lines with
+         |[] ->
+            ()
+         |head :: tail ->
+            mvwaddstr_safe win v_pos 2 head;
+            print_help_lines tail (succ v_pos)
+         end
+      in
+      begin match iface.abbrev_or_const with
+      |IsAbbrev ->
+         wattron win WA.bold;
+         mvwaddstr_safe win 5 0 "Abbreviations:";
+         wattroff win WA.bold;
+         mvwaddstr_safe win 6 1 "Common Functions:";
+         print_help_lines abbr_strings.functions 7;
+         mvwaddstr_safe win 13 1 "Change Modes:";
+         print_help_lines abbr_strings.modes 14;
+         mvwaddstr_safe win 17 1 "Miscellaneous:";
+         print_help_lines abbr_strings.misc 18;
+         mvwaddstr_safe win 20 1 ("execute abbreviation : " ^
+         try_find (Abbrev AbbrevEnter));
+         mvwaddstr_safe win 21 1 ("cancel abbreviation  : " ^
+         try_find (Abbrev AbbrevExit));
+      |IsConst ->
+         wattron win WA.bold;
+         mvwaddstr_safe win 5 0 "Constants:";
+         wattroff win WA.bold;
+         print_help_lines const_strings 7;
+         mvwaddstr_safe win 12 1 ("enter constant : " ^
+         try_find (Abbrev AbbrevEnter));
+      end;
+      assert (wnoutrefresh win)
+   end else begin
+      wattron win WA.bold;
+      assert (mvwaddstr win 5 0 "Matched Abbreviations:");
+      wattroff win WA.bold;
+      let highlight_len = String.length iface.abbrev_entry_buffer in
+      let rec draw_matches v_pos match_list =
+         if v_pos < iface.scr.hw_lines then
+            begin match match_list with
+            |[] ->
+               ()
+            |m :: tail ->
+               begin
+                  (* highlight the first 'highlight_len' characters *)
+                  wattron win WA.bold;
+                  mvwaddstr_safe win v_pos 2
+                     (Str.string_before m (highlight_len));
+                  wattroff win WA.bold;
+                  mvwaddstr_safe win v_pos (2 + highlight_len)
+                     (Str.string_after m (highlight_len));
+                  draw_matches (succ v_pos) tail
+               end
+            end
+         else
+            ()
+      in
+      draw_matches 6 iface.matched_abbrev_list;
+      assert (wnoutrefresh win)
+   end 
+
+
+(* draw help page in variable editing mode *)
+let draw_help_varedit iface win mvwaddstr_safe try_find =
+   wattron win WA.bold;
+   mvwaddstr_safe win 5 0 "Variable Mode Commands:";
+   wattroff win WA.bold;
+   mvwaddstr_safe win 6 2 ("enter variable   : " ^
+   try_find (VarEdit VarEditEnter));
+   mvwaddstr_safe win 7 2 ("complete variable: " ^
+   try_find (VarEdit VarEditComplete));
+   mvwaddstr_safe win 8 2 ("cancel entry     : " ^
+   try_find (VarEdit VarEditExit));
+   wattron win WA.bold;
+   mvwaddstr_safe win 10 0 "Matched variables:";
+   wattroff win WA.bold;
+   let highlight_len = 
+      begin match iface.completion with
+      |None   -> String.length iface.variable_entry_buffer 
+      |Some _ -> 0
+      end
+   in
+   let rec draw_matches v_pos match_list count =
+      if v_pos < iface.scr.hw_lines then
+         begin match match_list with
+         |[] ->
+            ()
+         |m :: tail ->
+            begin match iface.completion with
+            |None ->
+               (* highlight the first 'highlight_len' characters *)
+               wattron win WA.bold;
+               mvwaddstr_safe win v_pos 2
+                  (Str.string_before m (highlight_len));
+               wattroff win WA.bold;
+               mvwaddstr_safe win v_pos (2 + highlight_len)
+                  (Str.string_after m (highlight_len));
+            |Some num ->
+               (* highlight the entire selected match *)
+               if count = num then begin
+                  wattron win WA.bold;
+                  mvwaddstr_safe win v_pos 2 m;
+                  wattroff win WA.bold;
+               end else
+                  mvwaddstr_safe win v_pos 2 m;
+            end;
+            draw_matches (succ v_pos) tail (succ count)
+         end
+      else
+         ()
+   in
+   if List.length iface.matched_variables = 0 then
+      mvwaddstr_safe win 11 2 "(none)"
+   else
+      draw_matches 11 iface.matched_variables 0;
+   assert (wnoutrefresh win)
+
+
+(* draw help page in stack browsing mode *)
+let draw_help_browsing iface win mvwaddstr_safe try_find =
+   wattron win WA.bold;
+   mvwaddstr_safe win 5 0 "Browsing Operations:";
+   wattroff win WA.bold;
+   mvwaddstr_safe win 6 2  ("prev        : " ^
+   try_find (Browse PrevLine));
+   mvwaddstr_safe win 7 2  ("next        : " ^
+   try_find (Browse NextLine));
+   mvwaddstr_safe win 8 2  ("scroll left : " ^
+   try_find (Browse ScrollLeft));
+   mvwaddstr_safe win 9 2  ("scroll right: " ^
+   try_find (Browse ScrollRight));
+   mvwaddstr_safe win 10 2 ("roll down   : " ^
+   try_find (Browse RollDown));
+   mvwaddstr_safe win 11 2 ("roll up     : " ^
+   try_find (Browse RollUp));
+   mvwaddstr_safe win 12 2 ("dup         : " ^
+   try_find (Command Dup));
+   mvwaddstr_safe win 13 2 ("view        : " ^
+   try_find (Browse ViewEntry));
+   mvwaddstr_safe win 14 2 ("edit        : " ^
+   try_find (Browse EditEntry));
+   mvwaddstr_safe win 15 2 ("drop        : " ^
+   try_find (Browse Drop1));
+   mvwaddstr_safe win 16 2 ("dropn       : " ^
+   try_find (Browse DropN));
+   mvwaddstr_safe win 17 2 ("keep        : " ^
+   try_find (Browse Keep));
+   mvwaddstr_safe win 18 2 ("keepn       : " ^
+   try_find (Browse KeepN));
+   mvwaddstr_safe win 20 1 ("exit browsing mode: " ^
+   try_find (Browse EndBrowse));
+   assert (wnoutrefresh win)
+
+
+
 (* display the help window *)
 let draw_help (iface : interface_state_t) =
    let mvwaddstr_safe w vert horiz st =
@@ -417,6 +695,8 @@ let draw_help (iface : interface_state_t) =
    in
    let modes = iface.calc#get_modes () in
    begin match iface.scr.help_win with
+   |None ->
+      ()
    |Some win ->
       wclear win;
       wattron win WA.bold;
@@ -447,247 +727,28 @@ let draw_help (iface : interface_state_t) =
       |Rect -> "REC"
       |Polar -> "POL" in
       assert (mvwaddstr win 3 34 complex_str);
-      let try_find fn el =
-         try fn el
+      let try_find op =
+         try Rcfile.key_of_operation op
          with Not_found -> "(N/A)"
       in
-      (* FIXME: this could be done more cleanly... shouldn't have
-       * to look at two completely different modes *)
       begin match iface.interface_mode with
+      |StandardEditMode | UnitEditMode -> 
+         draw_help_standard iface win mvwaddstr_safe try_find
+      |IntEditMode ->
+         draw_help_intedit iface win mvwaddstr_safe try_find
+      |AbbrevEditMode ->
+         draw_help_abbrev iface win mvwaddstr_safe try_find
+      |VarEditMode ->
+         draw_help_varedit iface win mvwaddstr_safe try_find
       |BrowsingMode ->
-         wattron win WA.bold;
-         assert (mvwaddstr win 5 0 "Browsing Operations:");
-         wattroff win WA.bold;
-         mvwaddstr_safe win 6 2  ("prev        : " ^
-         try_find Rcfile.key_of_browse (Browse PrevLine));
-         mvwaddstr_safe win 7 2  ("next        : " ^
-         try_find Rcfile.key_of_browse (Browse NextLine));
-         mvwaddstr_safe win 8 2  ("scroll left : " ^
-         try_find Rcfile.key_of_browse (Browse ScrollLeft));
-         mvwaddstr_safe win 9 2  ("scroll right: " ^
-         try_find Rcfile.key_of_browse (Browse ScrollRight));
-         mvwaddstr_safe win 10 2 ("roll down   : " ^
-         try_find Rcfile.key_of_browse (Browse RollDown));
-         mvwaddstr_safe win 11 2 ("roll up     : " ^
-         try_find Rcfile.key_of_browse (Browse RollUp));
-         mvwaddstr_safe win 12 2 ("dup         : " ^
-         try_find Rcfile.key_of_command (Command Dup));
-         mvwaddstr_safe win 13 2 ("view        : " ^
-         try_find Rcfile.key_of_browse (Browse ViewEntry));
-         mvwaddstr_safe win 14 2 ("edit        : " ^
-         try_find Rcfile.key_of_browse (Browse EditEntry));
-         mvwaddstr_safe win 15 2 ("drop        : " ^
-         try_find Rcfile.key_of_browse (Browse Drop1));
-         mvwaddstr_safe win 16 2 ("dropn       : " ^
-         try_find Rcfile.key_of_browse (Browse DropN));
-         mvwaddstr_safe win 17 2 ("keep        : " ^
-         try_find Rcfile.key_of_browse (Browse Keep));
-         mvwaddstr_safe win 18 2 ("keepn       : " ^
-         try_find Rcfile.key_of_browse (Browse KeepN));
-         mvwaddstr_safe win 20 1 ("exit browsing mode: " ^
-         try_find Rcfile.key_of_browse (Browse EndBrowse));
-         assert (wnoutrefresh win)
-      |_ ->
-         begin match iface.help_mode with
-         |StandardHelp ->
-            if iface.help_page = 0 then begin
-               wattron win WA.bold;
-               assert (mvwaddstr win 5 0 "Common Operations:");
-               wattroff win WA.bold;
-               mvwaddstr_safe win 6 2  ("enter    : " ^
-               try_find Rcfile.key_of_edit (Edit Enter));
-               mvwaddstr_safe win 7 2  ("drop     : " ^
-               try_find Rcfile.key_of_command (Command Drop));
-               mvwaddstr_safe win 8 2  ("swap     : " ^
-               try_find Rcfile.key_of_command (Command Swap));
-               mvwaddstr_safe win 9 2  ("backspace: " ^
-               try_find Rcfile.key_of_edit (Edit Backspace));
-               mvwaddstr_safe win 10 2 ("add      : " ^
-               try_find Rcfile.key_of_function (Function Add));
-               mvwaddstr_safe win 11 2 ("subtract : " ^
-               try_find Rcfile.key_of_function (Function Sub));
-               mvwaddstr_safe win 12 2 ("multiply : " ^
-               try_find Rcfile.key_of_function (Function Mult));
-               mvwaddstr_safe win 13 2 ("divide   : " ^
-               try_find Rcfile.key_of_function (Function Div));
-               mvwaddstr_safe win 14 2 ("x^y      : " ^
-               try_find Rcfile.key_of_function (Function Pow));
-               mvwaddstr_safe win 15 2 ("negation : " ^
-               try_find Rcfile.key_of_function (Function Neg));
-               wattron win WA.bold;
-               mvwaddstr_safe win 16 0 "Miscellaneous:";
-               wattroff win WA.bold;
-               mvwaddstr_safe win 17 2 ("scientific notation     : " ^
-               try_find Rcfile.key_of_edit (Edit SciNotBase));
-               mvwaddstr_safe win 18 2 ("abbreviation entry mode : " ^
-               try_find Rcfile.key_of_command  (Command BeginAbbrev));
-               mvwaddstr_safe win 19 2 ("stack browsing mode     : " ^
-               try_find Rcfile.key_of_command (Command BeginBrowse));
-               mvwaddstr_safe win 20 2 ("refresh display         : " ^
-               try_find Rcfile.key_of_command (Command Refresh));
-               mvwaddstr_safe win 21 2 ("quit                    : " ^
-               try_find Rcfile.key_of_command (Command Quit));
-               assert (wnoutrefresh win)
-            end else begin
-               let adjust_len s len =
-                  if String.length s < len then
-                     s ^ (String.make (len - (String.length s)) ' ')
-                  else
-                     Str.string_before s len
-               in
-               let make_string colon_pos key_string abbr =
-                  (adjust_len key_string colon_pos) ^ ": " ^ abbr
-               in
-               wattron win WA.bold;
-               mvwaddstr_safe win 5 0 "Autobindings:";
-               wattroff win WA.bold;
-               if Array.length !Rcfile.autobind_keys <= 0 then
-                  mvwaddstr_safe win 6 2 "(none)"
-               else
-                  for i = 0 to pred (min (iface.scr.hw_lines - 6) (Array.length
-                  !Rcfile.autobind_keys)) do
-                     let (key, key_string, bound_f, age) = !Rcfile.autobind_keys.(i) in
-                     let abbr = match bound_f with
-                        |None    -> "(none)"
-                        |Some op -> Rcfile.abbrev_of_operation op
-                     in
-                     mvwaddstr_safe win (i + 6) 2 (make_string 12 key_string abbr)
-                  done;
-               assert (wnoutrefresh win)
-            end
-         |StandardIntHelp ->
-            wattron win WA.bold;
-            mvwaddstr_safe win 5 0 "Integer Editing Operations:";
-            wattroff win WA.bold;
-            mvwaddstr_safe win 6 2 ("enter    : " ^
-            try_find Rcfile.key_of_edit (Edit Enter));
-            mvwaddstr_safe win 7 2 ("set base : " ^
-            try_find Rcfile.key_of_edit (Edit SciNotBase));
-            mvwaddstr_safe win 8 2 ("cancel   : " ^
-            try_find Rcfile.key_of_intedit (IntEdit ExitIntEdit));
-            assert (wnoutrefresh win)
-         |AbbrevHelp ->
-            if String.length iface.abbrev_entry_buffer = 0 then
-               let abbr_strings = generate_abbrev_help () in
-               let rec print_help_lines lines v_pos =
-                  begin match lines with
-                  |[] ->
-                     ()
-                  |head :: tail ->
-                     mvwaddstr_safe win v_pos 2 head;
-                     print_help_lines tail (succ v_pos)
-                  end
-               in
-               begin
-                  wattron win WA.bold;
-                  mvwaddstr_safe win 5 0 "Abbreviations:";
-                  wattroff win WA.bold;
-                  mvwaddstr_safe win 6 1 "Common Functions:";
-                  print_help_lines abbr_strings.functions 7;
-                  mvwaddstr_safe win 13 1 "Change Modes:";
-                  print_help_lines abbr_strings.modes 14;
-                  mvwaddstr_safe win 17 1 "Miscellaneous:";
-                  print_help_lines abbr_strings.misc 18;
-                  mvwaddstr_safe win 20 1 ("execute abbreviation : " ^
-                  try_find Rcfile.key_of_abbrev (Abbrev EnterAbbrev));
-                  mvwaddstr_safe win 21 1 ("cancel abbreviation  : " ^
-                  try_find Rcfile.key_of_abbrev (Abbrev ExitAbbrev));
-                  assert (wnoutrefresh win)
-               end
-            else
-               begin
-                  wattron win WA.bold;
-                  assert (mvwaddstr win 5 0 "Matched Abbreviations:");
-                  wattroff win WA.bold;
-                  let highlight_len = String.length iface.abbrev_entry_buffer in
-                  let rec draw_matches v_pos match_list =
-                     if v_pos < iface.scr.hw_lines then
-                        begin match match_list with
-                        |[] ->
-                           ()
-                        |m :: tail ->
-                           begin
-                              (* highlight the first 'highlight_len' characters *)
-                              wattron win WA.bold;
-                              let len_str = String.length m in
-                              mvwaddstr_safe win v_pos 2
-                                 (Str.string_before m (highlight_len));
-                              wattroff win WA.bold;
-                              mvwaddstr_safe win v_pos (2 + highlight_len)
-                                 (Str.string_after m (highlight_len));
-                              draw_matches (succ v_pos) tail
-                           end
-                        end
-                     else
-                        ()
-                  in
-                  draw_matches 6 iface.matched_abbrev_entry_list;
-                  assert (wnoutrefresh win)
-               end 
-         |VarHelp ->
-            wattron win WA.bold;
-            mvwaddstr_safe win 5 0 "Variable Mode Commands:";
-            wattroff win WA.bold;
-            mvwaddstr_safe win 6 2 ("enter variable   : " ^
-            try_find Rcfile.key_of_varedit (VarEdit EnterVarEdit));
-            mvwaddstr_safe win 7 2 ("complete variable: " ^
-            try_find Rcfile.key_of_varedit (VarEdit CompleteVarEdit));
-            mvwaddstr_safe win 8 2 ("cancel entry     : " ^
-            try_find Rcfile.key_of_varedit (VarEdit ExitVarEdit));
-            wattron win WA.bold;
-            mvwaddstr_safe win 10 0 "Matched variables:";
-            wattroff win WA.bold;
-            let highlight_len = 
-               begin match iface.completion with
-               |None   -> String.length iface.variable_entry_buffer 
-               |Some _ -> 0
-               end
-            in
-            let rec draw_matches v_pos match_list count =
-               if v_pos < iface.scr.hw_lines then
-                  begin match match_list with
-                  |[] ->
-                     ()
-                  |m :: tail ->
-                     begin match iface.completion with
-                     |None ->
-                        (* highlight the first 'highlight_len' characters *)
-                        wattron win WA.bold;
-                        let len_str = String.length m in
-                        mvwaddstr_safe win v_pos 2
-                           (Str.string_before m (highlight_len));
-                        wattroff win WA.bold;
-                        mvwaddstr_safe win v_pos (2 + highlight_len)
-                           (Str.string_after m (highlight_len));
-                     |Some num ->
-                        (* highlight the entire selected match *)
-                        if count = num then begin
-                           wattron win WA.bold;
-                           mvwaddstr_safe win v_pos 2 m;
-                           wattroff win WA.bold;
-                        end else
-                           mvwaddstr_safe win v_pos 2 m;
-                     end;
-                     draw_matches (succ v_pos) tail (succ count)
-                  end
-               else
-                  ()
-            in
-            if List.length iface.matched_variables = 0 then
-               mvwaddstr_safe win 11 2 "(none)"
-            else
-               draw_matches 11 iface.matched_variables 0;
-            assert (wnoutrefresh win)
-         end 
+         draw_help_browsing iface win mvwaddstr_safe try_find
       end
-   |None ->
-      ()
    end; 
    assert (wmove iface.scr.entry_win (iface.scr.ew_lines - 1) (iface.scr.ew_cols - 1))
 
 
 let draw_message (iface : interface_state_t) msg =
-   draw_update_stack iface;
+   draw_stack iface;
    let error_lines = Utility.wordwrap msg (iface.scr.sw_cols-2) in
    let trunc_error_lines = 
       if List.length error_lines > 4 then
