@@ -1,10 +1,9 @@
-(*  Orpie -- a stack-based RPN calculator for the console
- *  Copyright (C) 2003-2004  Paul Pelzl
+(*  Orpie -- a fullscreen RPN calculator for the console
+ *  Copyright (C) 2003-2004, 2005, 2006-2007 Paul Pelzl
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  it under the terms of the GNU General Public License, Version 2,
+ *  as published by the Free Software Foundation.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -19,6 +18,10 @@
  *  <pelzlpj@eecs.umich.edu>.
  *)
 
+(* interface_main.ml
+ * This file has the bulk of the implementation of the curses-based interface,
+ * including the main program loop that grabs keypresses and processes them.
+ * The screen rendering code is found in interface_draw.ml . *)
 
 open Curses;;
 open Printf;;
@@ -147,7 +150,6 @@ let handle_resize (iface : interface_state_t) =
    (* reset ncurses *)
    endwin ();
    assert (refresh ());
-   let rows, cols = get_size () in
    resize_subwins iface.scr;
    handle_refresh iface;;
 
@@ -205,8 +207,8 @@ let get_entry_from_buffer (iface : interface_state_t) =
          let buffer = iface.gen_buffer.(0) in
          try
             let ff = get_float_el buffer.re_mantissa buffer.re_exponent in
-            let fu = Units.unit_of_float_string ff iface.units_entry_buffer in
-            RpcFloatUnit fu
+            let uu = Units.units_of_string iface.units_entry_buffer !Rcfile.unit_table in
+            RpcFloatUnit (ff, uu)
          with 
             |Failure "float_of_string" ->
                raise (Invalid_argument "improperly formatted floating-point data")
@@ -220,12 +222,15 @@ let get_entry_from_buffer (iface : interface_state_t) =
             begin match buffer.is_polar with
             |false ->
                let real_part = get_float_el buffer.re_mantissa 
-               buffer.re_exponent
+                  buffer.re_exponent
                and imag_part = get_float_el buffer.im_mantissa
-               buffer.im_exponent in
-               let cu = Units.unit_of_cpx_string 
-               {re = real_part; im = imag_part} iface.units_entry_buffer in
-               RpcComplexUnit cu
+                  buffer.im_exponent 
+               in
+               let cc = {re = real_part; im = imag_part}
+               and uu = Units.units_of_string iface.units_entry_buffer 
+                  !Rcfile.unit_table
+               in
+               RpcComplexUnit (cc, uu)
             (* if is_polar==true, then the data in the buffer represents
              * polar data... so convert it to rect notation before storing
              * it as a complex number. *)
@@ -240,10 +245,11 @@ let get_entry_from_buffer (iface : interface_state_t) =
                      (pi /. 180.0 *. (get_float_el buffer.im_mantissa
                      buffer.im_exponent))
                in
-               let cu = Units.unit_of_cpx_string 
-               {re = r *. (cos theta); im = r *. (sin theta)}
-               iface.units_entry_buffer in
-               RpcComplexUnit cu
+               let cc = {re = r *. (cos theta); im = r *. (sin theta)}
+               and uu = Units.units_of_string iface.units_entry_buffer
+                  !Rcfile.unit_table
+               in
+               RpcComplexUnit (cc, uu)
             end
          with 
             |Failure "float_of_string" ->
@@ -265,7 +271,9 @@ let get_entry_from_buffer (iface : interface_state_t) =
                temp_arr.(i) <- (get_float_el iface.gen_buffer.(i).re_mantissa
                   iface.gen_buffer.(i).re_exponent)
             done;
-            let uu = Units.unit_of_string iface.units_entry_buffer in
+            let uu = Units.units_of_string iface.units_entry_buffer
+               !Rcfile.unit_table
+            in
             RpcFloatMatrixUnit (Gsl_matrix.of_array temp_arr matrix_rows
             iface.matrix_cols, uu)
          with Failure "float_of_string" ->
@@ -310,7 +318,9 @@ let get_entry_from_buffer (iface : interface_state_t) =
                   im = r *. (sin theta)}
                end
             done;
-            let uu = Units.unit_of_string iface.units_entry_buffer in
+            let uu = Units.units_of_string iface.units_entry_buffer
+               !Rcfile.unit_table 
+            in
             RpcComplexMatrixUnit (Gsl_matrix_complex.of_array temp_arr
             matrix_rows iface.matrix_cols, uu)
          with Failure "float_of_string" ->
@@ -366,19 +376,13 @@ let push_entry (iface : interface_state_t) =
 
 (* handle an 'enter' keypress *)
 let handle_enter (iface : interface_state_t) =
-   iface.interface_mode <- StandardEntryMode;
-   begin match iface.help_mode with
-   |StandardIntHelp ->
-      iface.help_mode <- StandardHelp;
-      draw_help iface
-   |_ ->
-      ()
-   end;
+   iface.interface_mode <- StandardEditMode;
+   draw_help iface;
    try
-      (if iface.has_entry then
+      if iface.has_entry then
          push_entry iface
       else
-         raise Not_handled);
+         raise Not_handled;
       draw_update_stack iface
    with Invalid_argument error_msg ->
       draw_error iface error_msg;
@@ -391,7 +395,6 @@ let handle_begin_int (iface : interface_state_t) =
       (iface.entry_type <- IntEntry;
       iface.interface_mode <- IntEditMode;
       iface.int_entry_buffer <- "";
-      iface.help_mode <- StandardIntHelp;
       draw_help iface;
       draw_update_entry iface)
    else
@@ -526,9 +529,8 @@ let handle_angle (iface : interface_state_t) =
 
 (* cancel integer entry mode *)
 let handle_exit_int (iface : interface_state_t) =
-   iface.interface_mode <- StandardEntryMode;
+   iface.interface_mode <- StandardEditMode;
    iface.entry_type <- FloatEntry;
-   iface.help_mode <- StandardHelp;
    iface.is_entering_base <- false;
    iface.int_base_string <- "";
    if (String.length iface.gen_buffer.(0).re_mantissa > 0) or
@@ -909,9 +911,20 @@ let handle_digit (iface : interface_state_t) key =
 
 (* begin abbrev entry *)
 let handle_begin_abbrev (iface : interface_state_t) =
-   if iface.interface_mode <> AbbrevEntryMode then begin
-      iface.interface_mode <- AbbrevEntryMode;
-      iface.help_mode <- AbbrevHelp;
+   if iface.interface_mode <> AbbrevEditMode then begin
+      iface.interface_mode <- AbbrevEditMode;
+      iface.abbrev_or_const <- IsAbbrev;
+      draw_help iface;
+      draw_update_entry iface
+      (* do other cleanup stuff *)
+   end else
+      ()
+
+(* begin constant entry *)
+let handle_begin_const (iface : interface_state_t) =
+   if iface.interface_mode <> AbbrevEditMode then begin
+      iface.interface_mode <- AbbrevEditMode;
+      iface.abbrev_or_const <- IsConst;
       draw_help iface;
       draw_update_entry iface
       (* do other cleanup stuff *)
@@ -924,7 +937,6 @@ let handle_begin_variable (iface : interface_state_t) =
    if iface.interface_mode <> VarEditMode then begin
       iface.interface_mode <- VarEditMode;
       iface.entry_type <- VarEntry;
-      iface.help_mode <- VarHelp;
       (* FIXME: generation of the sorted variable list can be done far
        * more efficiently *)
       let add_variable var_str var_binding var_list =
@@ -997,7 +1009,7 @@ let handle_end_browse (iface : interface_state_t) =
    iface.horiz_scroll <- 0;
    iface.stack_selection <- 1;
    iface.stack_bottom_row <- 1;
-   iface.interface_mode <- StandardEntryMode;
+   iface.interface_mode <- StandardEditMode;
    draw_help iface;
    draw_update_stack iface
    
@@ -1149,7 +1161,6 @@ let edit_parse_input_textfile (iface : interface_state_t) (is_browsing) =
       in ();
       (* wake up curses again, and check for resize *)
       assert (refresh ());
-      let rows, cols = get_size () in
       resize_subwins iface.scr;
       handle_refresh iface;
       let edited_buf = Utility.expand_open_in_ascii input_file in
@@ -1164,23 +1175,15 @@ let edit_parse_input_textfile (iface : interface_state_t) (is_browsing) =
             Txtin_parser.decode_data_deg Txtin_lexer.token lexbuf
          end
       in
-      let rec push_data d =
-         begin match d with
-         |[] -> 
-            ()
-         |hd :: tl -> 
-            if is_browsing then
-               begin
-                  iface.calc#delete iface.stack_selection;
-                  iface.calc#push hd;
-                  iface.calc#rolldown iface.stack_selection
-               end
-            else
-               iface.calc#push hd;
-            push_data tl
-         end
+      let push_data el =
+         if is_browsing then begin
+            iface.calc#delete iface.stack_selection;
+            iface.calc#push el;
+            iface.calc#rolldown iface.stack_selection
+         end else
+            iface.calc#push el;
       in
-      push_data data;
+      List.iter push_data data;
       close_in edited_buf;
       handle_refresh iface
    with
@@ -1264,7 +1267,7 @@ let handle_edit_input (iface : interface_state_t) =
 (* display an "about" screen *)
 let handle_about (iface : interface_state_t) =
    draw_about iface;
-   let a = getch () in ();
+   let _ = getch () in ();
    erase ();
    assert (refresh ());
    handle_refresh iface
@@ -1282,7 +1285,7 @@ let handle_cycle_help (iface : interface_state_t) =
 
 (* quit the calculator *)
 let handle_quit (iface : interface_state_t) =
-   iface.calc#save_state ();
+   Statefile.save_state (iface.calc#get_state ());
    iface.run_calc <- false
 
 
@@ -1531,6 +1534,8 @@ let process_command (iface : interface_state_t) cc =
       handle_begin_browse iface
    |BeginAbbrev ->
       handle_begin_abbrev iface
+   |BeginConst ->
+      handle_begin_const iface
    |BeginVar ->
       handle_begin_variable iface
    |Quit ->
@@ -1598,128 +1603,122 @@ let process_command (iface : interface_state_t) cc =
 
 
 (****************************************************************)
-(* IMPLEMENTATION OF ABBREVIATION ENTRY SYSTEM                  *)
+(* IMPLEMENTATION OF ABBREVIATION AND CONSTANT ENTRY SYSTEM     *)
 (****************************************************************)
 
 
 
 (* exit abbrev entry *)
 let handle_exit_abbrev (iface : interface_state_t) =
-   if iface.interface_mode = AbbrevEntryMode then
-      (iface.interface_mode <- StandardEntryMode;
-      iface.help_mode <- StandardHelp;
+   if iface.interface_mode = AbbrevEditMode then begin
+      iface.interface_mode <- StandardEditMode;
       iface.abbrev_entry_buffer <- "";
-      iface.matched_abbrev_entry <- "";
       draw_help iface;
-      draw_update_entry iface)
-   else
+      draw_update_entry iface
+   end else
       ()
 
 
 (* search through a list of commands and find all that match
- * iface.abbrev_entry_buffer.  As a side effect, set matched_abbrev_entry
- * to the head of this list.
+ * iface.abbrev_entry_buffer.
  * The list is built up in reverse order using Str.search_backward,
  * so the head of the list is actually the first match. *)
 let match_abbrev_buffer (iface : interface_state_t) buf =
    if String.length buf > 0 then
-      (let regex_str = "^" ^ (Str.quote buf) ^ ".*$" in
-      let regex = Str.regexp regex_str in
-      let rec find_matching_strings starting_pos matches_list =
-         try
-            let next_pos = 
-               Str.search_backward regex !Rcfile.abbrev_commands starting_pos
-            in
-            let m = Str.matched_string !Rcfile.abbrev_commands in
-            if next_pos >= 1 then
-               find_matching_strings (pred next_pos) (m :: matches_list)
-            else
-               (m :: matches_list)
-         with
-            Not_found ->
-               begin
-                  match matches_list with
-                  |[] -> raise Not_found
-                  |_ -> matches_list
-               end
+      let regex = Str.regexp_string buf in
+      let find_matches prev_matches el =
+         if Str.string_match regex el 0 then
+            el :: prev_matches
+         else
+            prev_matches
       in
-      iface.matched_abbrev_entry <- "";
-      let m_list =
-         find_matching_strings (pred (String.length !Rcfile.abbrev_commands)) [];
+      let match_list =
+         match iface.abbrev_or_const with
+         |IsAbbrev ->
+            List.fold_left find_matches [] !Rcfile.abbrev_commands
+         |IsConst ->
+            List.fold_left find_matches [] !Rcfile.constant_symbols
       in
-      iface.matched_abbrev_entry <- List.hd m_list;
-      m_list)
-   else
-      (iface.matched_abbrev_entry <- "";
-      raise Not_found)
+      if List.length match_list = 0 then raise Not_found
+      else match_list 
+   else 
+      raise Not_found
 
 
 (* backspace during abbrev entry *)
 let handle_abbrev_backspace (iface : interface_state_t) =
    let len = String.length iface.abbrev_entry_buffer in
-   if len > 0 then
-      (iface.abbrev_entry_buffer <- Str.string_before iface.abbrev_entry_buffer 
+   if len > 0 then begin
+      iface.abbrev_entry_buffer <- Str.string_before iface.abbrev_entry_buffer 
       (pred len);
-      (try 
-         iface.matched_abbrev_entry_list <- 
-            match_abbrev_buffer iface iface.abbrev_entry_buffer
+      begin try 
+         iface.matched_abbrev_list <- 
+         match_abbrev_buffer iface iface.abbrev_entry_buffer
       with 
-         Not_found -> ());
-      (if len = 1 then
-         iface.matched_abbrev_entry <- ""
-      else
-         ());
+         Not_found -> ()
+      end;
       draw_help iface;
-      draw_update_entry iface)
-   else
+      draw_update_entry iface
+   end else
       ()
 
 
 (* handle entry of an arbitrary character in abbrev mode *)
 let handle_abbrev_character (iface : interface_state_t) key =
-   let ch = char_of_int key in
-   let test_buffer = iface.abbrev_entry_buffer ^ (String.make 1 ch) in
-   (* search through the list of commands for the first one that matches
-    * iface.abbrev_entry_buffer *)
    try
-      iface.matched_abbrev_entry_list <- match_abbrev_buffer iface test_buffer;
+      let ch = char_of_int key in
+      let test_buffer = iface.abbrev_entry_buffer ^ (String.make 1 ch) in
+      (* search through the list of commands for the first one that matches
+       * iface.abbrev_entry_buffer *)
+      iface.matched_abbrev_list <- match_abbrev_buffer iface test_buffer;
       iface.abbrev_entry_buffer <- test_buffer;
       draw_help iface;
       draw_update_entry iface
    with
-      Not_found -> let err = beep () in ()
+      Not_found | Invalid_argument "char_of_int" -> let _ = beep () in ()
 
 
 (* enter an abbrev entry *)
 let handle_enter_abbrev (iface : interface_state_t) =
-   if iface.interface_mode = AbbrevEntryMode then
-      (iface.interface_mode <- StandardEntryMode;
-      iface.help_mode <- StandardHelp;
-      (try
-         iface.matched_abbrev_entry_list <- 
+   if iface.interface_mode = AbbrevEditMode then begin
+      iface.interface_mode <- StandardEditMode;
+      begin try
+         iface.matched_abbrev_list <- 
             match_abbrev_buffer iface iface.abbrev_entry_buffer;
-         let operation = Rcfile.translate_abbrev
-         iface.matched_abbrev_entry in
-         begin match operation with
-         |Function ff -> 
-            process_function iface ff
-         |Command cc  -> process_command iface cc
-         |_ -> failwith 
-            "found abbrev command that is neither Function nor Command"
-         end;
-         (* check whether ff should be autobound *)
-         begin try
-            let _ = Rcfile.key_of_operation operation in ()
-         with Not_found ->
-            register_autobinding operation
+         let first_abbrev_match = 
+            if iface.matched_abbrev_list = [] then ""
+            else List.hd iface.matched_abbrev_list
+         in
+         begin match iface.abbrev_or_const with
+         |IsAbbrev ->
+            let operation = Rcfile.translate_abbrev first_abbrev_match in
+            begin match operation with
+            |Function ff -> 
+               process_function iface ff
+            |Command cc  -> 
+               process_command iface cc
+            |_ -> 
+               failwith 
+               "found abbrev command that is neither Function nor Command"
+            end;
+            (* check whether ff should be autobound *)
+            begin try
+               let _ = Rcfile.key_of_operation operation in ()
+            with Not_found ->
+               register_autobinding operation
+            end
+         |IsConst ->
+            let const = Rcfile.translate_constant first_abbrev_match in
+            iface.calc#enter_const const.Units.coeff const.Units.comp_units;
+            draw_update_stack iface
          end
       with
-         Not_found -> ());
+         Not_found -> ()
+      end;
       iface.abbrev_entry_buffer <- "";
-      iface.matched_abbrev_entry <- "";
       draw_help iface;
-      draw_update_entry iface)
-   else
+      draw_update_entry iface
+   end else
       ()
 
 
@@ -1730,8 +1729,7 @@ let handle_enter_abbrev (iface : interface_state_t) =
 (* exit variable entry *)
 let handle_exit_variable (iface : interface_state_t) =
    if iface.interface_mode = VarEditMode then begin
-      iface.interface_mode <- StandardEntryMode;
-      iface.help_mode <- StandardHelp;
+      iface.interface_mode <- StandardEditMode;
       iface.entry_type <- FloatEntry;
       iface.variable_entry_buffer <- "";
       draw_help iface;
@@ -1743,18 +1741,14 @@ let handle_exit_variable (iface : interface_state_t) =
 (* search through a list of variables and find all that match
  * iface.variable_entry_buffer. *)
 let match_variable_buffer (iface : interface_state_t) buf =
-   let buf_regex = Str.regexp (Str.quote buf) in
-   let rec match_aux var_lst matches_lst =
-      match var_lst with
-      |[] ->
-         matches_lst
-      |vv :: tail ->
-         if Str.string_match buf_regex vv 0 then
-            match_aux tail (vv :: matches_lst)
-         else
-            match_aux tail matches_lst
+   let buf_regex = Str.regexp_string buf in
+   let find_matches prev_result el =
+      if Str.string_match buf_regex el 0 then
+         el :: prev_result
+      else
+         prev_result
    in
-   List.rev (match_aux iface.sorted_variables [])
+   List.fold_left find_matches [] (List.rev iface.sorted_variables)
 
 
 
@@ -1811,8 +1805,7 @@ let handle_enter_variable (iface : interface_state_t) =
          push_entry iface
       else
          iface.entry_type <- FloatEntry;
-      iface.interface_mode <- StandardEntryMode;
-      iface.help_mode <- StandardHelp;
+      iface.interface_mode <- StandardEditMode;
       iface.completion <- None;
       iface.variable_entry_buffer <- "";
       draw_help iface;
@@ -1855,7 +1848,7 @@ let handle_complete_variable (iface : interface_state_t) =
 (* exit units entry *)
 let handle_exit_units (iface : interface_state_t) =
    if iface.interface_mode = UnitEditMode then begin
-      iface.interface_mode <- StandardEntryMode;
+      iface.interface_mode <- StandardEditMode;
       iface.units_entry_buffer <- "";
       iface.is_entering_units <- false;
       draw_update_entry iface
@@ -1871,7 +1864,7 @@ let handle_units_backspace (iface : interface_state_t) =
       (pred len);
       draw_update_entry iface
    end else begin
-      iface.interface_mode <- StandardEntryMode;
+      iface.interface_mode <- StandardEditMode;
       iface.is_entering_units <- false;
       draw_update_entry iface
    end
@@ -1879,20 +1872,207 @@ let handle_units_backspace (iface : interface_state_t) =
 
 (* handle entry of an arbitrary character in units entry mode *)
 let handle_units_character (iface : interface_state_t) key =
-   let allowable_regex = Str.regexp "[a-zA-Z0-9\\.\\^\\*/-]" in
-   let ch = char_of_int key in
-   let ss = String.make 1 ch in
-   if Str.string_match allowable_regex ss 0 then begin
-      iface.units_entry_buffer <- iface.units_entry_buffer ^ ss;
-      draw_update_entry iface
-   end else
-      let _ = beep () in ()
+   try
+      let allowable_regex = Str.regexp "[a-zA-Z0-9\\.\\^\\*/-]" in
+      let ch = char_of_int key in
+      let ss = String.make 1 ch in
+      if Str.string_match allowable_regex ss 0 then begin
+         iface.units_entry_buffer <- iface.units_entry_buffer ^ ss;
+         draw_update_entry iface
+      end else
+         let _ = beep () in ()
+   with
+      Invalid_argument "char_of_int" -> let _ = beep () in ()
 
 
 
 (****************************************************************)
 (* MAIN LOOP                                                    *)
 (****************************************************************)
+
+(* handle a keypress in standard entry mode *)
+let handle_keypress_standard key (iface : interface_state_t) =
+   (* check whether this keypress is a macro *)
+   begin try
+      let ch_list = Rcfile.macro_of_key key in
+      let rec push_macro chars =
+         match chars with
+         | [] -> ()
+         | head :: tail ->
+            let _ = ungetch head in 
+            push_macro tail
+      in
+      push_macro ch_list
+   with Not_found ->
+   (* if it's not a macro, go ahead and process it *)
+      begin try
+      (* editing operations take priority *)
+         let edit_op = Rcfile.edit_of_key key in
+         begin match edit_op with
+         |Digit ->
+            handle_digit iface key
+         |Enter ->
+            handle_enter iface
+         |Backspace ->
+            handle_backspace iface
+         |Minus ->
+            handle_minus iface
+         |BeginInteger ->
+            handle_begin_int iface
+         |BeginComplex ->
+            handle_begin_complex iface
+         |BeginMatrix ->
+            handle_begin_matrix iface
+         |Separator ->
+            handle_separator iface
+         |Angle ->
+            handle_angle iface
+         |SciNotBase ->
+            handle_scientific_notation iface
+         |BeginUnits ->
+            handle_begin_units iface
+         end
+      with Not_found | Not_handled ->
+         (* next we try to match on functions *)
+         try 
+            let function_op = Rcfile.function_of_key key in
+            process_function iface function_op
+         with Not_found ->
+            if iface.has_entry then
+               (* finally we try entry of digits *)
+               handle_digit iface key
+            else
+               (* commands are only suitable when there is no entry *)
+               try 
+                  let command_op = Rcfile.command_of_key key in
+                  process_command iface command_op
+               with Not_found ->
+                  handle_digit iface key
+      end
+   end
+
+
+(* handle a keypress in integer editing mode *)
+let handle_keypress_intedit key (iface : interface_state_t) =
+   begin try 
+      let edit_op = Rcfile.edit_of_key key in
+      begin match edit_op with
+      |Digit ->
+         handle_digit iface key
+      |Enter ->
+         handle_enter iface
+      |Backspace ->
+         handle_backspace iface
+      |Minus ->
+         handle_minus iface
+      |SciNotBase ->
+         handle_scientific_notation iface
+      |_ -> raise Not_handled
+      end
+   with Not_found | Not_handled ->
+      try
+         let intedit_op = Rcfile.intedit_of_key key in
+         begin match intedit_op with
+         |IntEditExit ->
+            handle_exit_int iface
+         end
+      with Not_found | Not_handled ->
+         handle_digit iface key
+   end
+
+
+(* handle a keypress in units entry mode *)
+let handle_keypress_unitedit key (iface : interface_state_t) =
+   begin try
+      let edit_op = Rcfile.edit_of_key key in
+      begin match edit_op with
+      |Enter ->
+         handle_enter iface
+      |Backspace ->
+         handle_units_backspace iface
+      |_ -> 
+         handle_units_character iface key
+      end
+   with Not_found | Not_handled ->
+      handle_units_character iface key
+   end
+
+
+(* handle a keypress in abbrev entry mode *)
+let handle_keypress_abbrev key (iface : interface_state_t) =
+   begin try
+      let abbrev_op = Rcfile.abbrev_of_key key in
+      begin match abbrev_op with
+      |AbbrevExit ->
+         handle_exit_abbrev iface
+      |AbbrevEnter ->
+         handle_enter_abbrev iface
+      |AbbrevBackspace ->
+         handle_abbrev_backspace iface
+      end
+   with Not_found ->
+      handle_abbrev_character iface key
+   end
+
+
+(* handle a keypress in variable editing mode *)
+let handle_keypress_varedit key (iface : interface_state_t) =
+   begin try
+      let varedit_op = Rcfile.varedit_of_key key in
+      begin match varedit_op with
+      |VarEditExit ->
+         handle_exit_variable iface
+      |VarEditEnter ->
+         handle_enter_variable iface
+      |VarEditBackspace ->
+         handle_variable_backspace iface
+      |VarEditComplete ->
+         handle_complete_variable iface
+      end
+   with Not_found ->
+      handle_variable_character iface key
+   end
+
+
+(* handle a keypress in stack browsing mode *)
+let handle_keypress_browse key (iface : interface_state_t) =
+   try
+      let browse_op = Rcfile.browse_of_key key in
+      begin match browse_op with
+      |EndBrowse ->
+         handle_end_browse iface
+      |ScrollLeft ->
+         handle_scroll_left iface
+      |ScrollRight ->
+         handle_scroll_right iface
+      |RollDown ->
+         handle_rolldown iface
+      |RollUp ->
+         handle_rollup iface
+      |PrevLine ->
+         handle_prev_line iface
+      |NextLine ->
+         handle_next_line iface
+      |Echo ->
+         handle_browse_echo iface
+      |ViewEntry ->
+         handle_browse_view iface
+      |Drop1 ->
+         handle_browse_drop1 iface
+      |DropN ->
+         handle_browse_dropn iface
+      |Keep ->
+         handle_browse_keep iface
+      |KeepN ->
+         handle_browse_keepn iface
+      |EditEntry ->
+         handle_browse_edit iface
+      end
+   with Not_found | Not_handled ->
+      ()
+
+
+
 let do_main_loop (iface : interface_state_t) =
    while iface.run_calc do
       iface.calc#launch_fill_in_thread ();
@@ -1902,212 +2082,12 @@ let do_main_loop (iface : interface_state_t) =
          handle_resize iface
       else
          match iface.interface_mode with
-         |StandardEntryMode ->
-            (* check whether this keypress is a macro *)
-            begin try
-               let ch_list = Rcfile.macro_of_key key in
-               let rec push_macro chars =
-                  match chars with
-                  | [] -> ()
-                  | head :: tail ->
-                     let _ = ungetch head in 
-                     push_macro tail
-               in
-               push_macro ch_list
-            with Not_found ->
-            (* if it's not a macro, go ahead and process it *)
-            begin try
-            (* editing operations take priority *)
-               let edit_op = Rcfile.edit_of_key key in
-               match edit_op with
-               |Edit ee ->
-                  begin match ee with
-                  |Digit ->
-                     handle_digit iface key
-                  |Enter ->
-                     handle_enter iface
-                  |Backspace ->
-                     handle_backspace iface
-                  |Minus ->
-                     handle_minus iface
-                  |BeginInteger ->
-                     handle_begin_int iface
-                  |BeginComplex ->
-                     handle_begin_complex iface
-                  |BeginMatrix ->
-                     handle_begin_matrix iface
-                  |Separator ->
-                     handle_separator iface
-                  |Angle ->
-                     handle_angle iface
-                  |SciNotBase ->
-                     handle_scientific_notation iface
-                  |BeginUnits ->
-                     handle_begin_units iface
-                  end
-               |_ ->
-                  failwith "Non-Edit operation found in Edit Hashtbl"
-            with Not_found | Not_handled ->
-               (* next we try to match on functions *)
-               try 
-                  let function_op = Rcfile.function_of_key key in
-                  match function_op with
-                  |Function ff ->
-                     process_function iface ff
-                  |_ ->
-                     failwith "Non-Function operation found in Function Hashtbl"
-               with Not_found ->
-                  if iface.has_entry then
-                     (* finally we try entry of digits *)
-                     handle_digit iface key
-                  else
-                     (* commands are only suitable when there is no entry *)
-                     try 
-                        let command_op = Rcfile.command_of_key key in
-                        match command_op with
-                        |Command cc ->
-                           process_command iface cc
-                        |_ ->
-                           failwith "Non-Command operation found in Command Hashtbl"
-                     with Not_found ->
-                        handle_digit iface key
-            end
-            end
-         |IntEditMode ->
-            begin 
-               try 
-                  let edit_op = Rcfile.edit_of_key key in
-                  match edit_op with
-                  |Edit ee ->
-                     begin match ee with
-                     |Digit ->
-                        handle_digit iface key
-                     |Enter ->
-                        handle_enter iface
-                     |Backspace ->
-                        handle_backspace iface
-                     |Minus ->
-                        handle_minus iface
-                     |SciNotBase ->
-                        handle_scientific_notation iface
-                     |_ -> raise Not_handled
-                     end
-                  |_ ->
-                     failwith "Non-Edit operation found in Edit Hashtbl"
-               with Not_found | Not_handled ->
-                  try
-                     let intedit_op = Rcfile.intedit_of_key key in
-                     match intedit_op with
-                     |IntEdit ie ->
-                        begin match ie with
-                        |ExitIntEdit ->
-                           handle_exit_int iface
-                        end
-                     |_ ->
-                        failwith "Non-IntEdit operation found in IntEdit Hashtbl"
-                  with Not_found | Not_handled ->
-                     handle_digit iface key
-            end
-         |UnitEditMode ->
-            begin try
-               let edit_op = Rcfile.edit_of_key key in
-               match edit_op with
-               |Edit ee ->
-                  begin match ee with
-                  |Enter ->
-                     handle_enter iface
-                  |Backspace ->
-                     handle_units_backspace iface
-                  |_ -> 
-                     handle_units_character iface key
-                  end
-               |_ ->
-                  failwith "Non-Edit operation found in Edit Hashtbl"
-            with Not_found | Not_handled ->
-               handle_units_character iface key
-            end
-         |AbbrevEntryMode ->
-            begin
-            (* check to see whether the user is either exiting abbrev mode or
-             * is applying the abbrev command *)
-            try
-               let abbrev_op = Rcfile.abbrev_of_key key in
-               match abbrev_op with
-               |Abbrev ee ->
-                  begin
-                     match ee with
-                     |ExitAbbrev ->
-                        handle_exit_abbrev iface
-                     |EnterAbbrev ->
-                        handle_enter_abbrev iface
-                     |AbbrevBackspace ->
-                        handle_abbrev_backspace iface
-                  end
-               |_ ->
-                  failwith "Non-Abbrev command found in Abbrev Hashtbl"
-            with Not_found ->
-               handle_abbrev_character iface key
-            end
-         |VarEditMode ->
-            begin try
-               let varedit_op = Rcfile.varedit_of_key key in
-               match varedit_op with
-               |VarEdit vv ->
-                  begin match vv with
-                  |ExitVarEdit ->
-                     handle_exit_variable iface
-                  |EnterVarEdit ->
-                     handle_enter_variable iface
-                  |VarEditBackspace ->
-                     handle_variable_backspace iface
-                  |CompleteVarEdit ->
-                     handle_complete_variable iface
-                  end
-               |_ ->
-                  failwith "Non-Variable command found in VarEdit Hashtbl"
-            with Not_found ->
-               handle_variable_character iface key
-            end
-         |BrowsingMode ->
-            try
-               let browse_op = Rcfile.browse_of_key key in
-               match browse_op with
-               |Browse bb ->
-                  begin
-                     match bb with
-                     |EndBrowse ->
-                        handle_end_browse iface
-                     |ScrollLeft ->
-                        handle_scroll_left iface
-                     |ScrollRight ->
-                        handle_scroll_right iface
-                     |RollDown ->
-                        handle_rolldown iface
-                     |RollUp ->
-                        handle_rollup iface
-                     |PrevLine ->
-                        handle_prev_line iface
-                     |NextLine ->
-                        handle_next_line iface
-                     |Echo ->
-                        handle_browse_echo iface
-                     |ViewEntry ->
-                        handle_browse_view iface
-                     |Drop1 ->
-                        handle_browse_drop1 iface
-                     |DropN ->
-                        handle_browse_dropn iface
-                     |Keep ->
-                        handle_browse_keep iface
-                     |KeepN ->
-                        handle_browse_keepn iface
-                     |EditEntry ->
-                        handle_browse_edit iface
-                  end
-               |_ ->
-                  failwith "Non-Browsing operation found in Browse Hashtbl"
-            with Not_found | Not_handled ->
-               ()
+         |StandardEditMode -> handle_keypress_standard key iface
+         |IntEditMode      -> handle_keypress_intedit key iface
+         |AbbrevEditMode   -> handle_keypress_abbrev key iface
+         |VarEditMode      -> handle_keypress_varedit key iface
+         |UnitEditMode     -> handle_keypress_unitedit key iface
+         |BrowsingMode     -> handle_keypress_browse key iface
    done
 
 
@@ -2125,7 +2105,7 @@ let run (iface : interface_state_t) =
 
    begin
       try
-         iface.calc#load_state ();
+         iface.calc#set_state (Statefile.load_state ());
          draw_stack iface;
          draw_help iface;
          draw_update_entry iface;
