@@ -1,10 +1,9 @@
-(*  Orpie -- a stack-based RPN calculator for the console
- *  Copyright (C) 2003-2004  Paul Pelzl
+(*  Orpie -- a fullscreen RPN calculator for the console
+ *  Copyright (C) 2003-2004, 2005, 2006-2007 Paul Pelzl
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  it under the terms of the GNU General Public License, Version 2,
+ *  as published by the Free Software Foundation.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -19,6 +18,10 @@
  *  <pelzlpj@eecs.umich.edu>.
  *)
 
+(* rcfile.ml
+ * This file includes everything associated with processing the orpierc file.
+ * In particular, this includes a number of hashtables used to store the
+ * bindings of curses keypresses to calculator operations. *)
 
 open Genlex;;
 open Curses;;
@@ -61,6 +64,8 @@ let autobind_keys_list : (int * string * operation_t option * int) list ref = re
 let autobind_keys = ref (Array.make 1 (0, "", None, 0));;
 (* List of included rc files *)
 let included_rcfiles : (string list) ref = ref [];;
+(* Unit definition table *)
+let unit_table = ref Units.empty_unit_table;;
 
 
 let function_of_key key =
@@ -97,42 +102,91 @@ let key_of_varedit edit_op =
 
 let key_of_operation (op : operation_t) =
    match op with
-   |Function _ -> Hashtbl.find table_function_key op
-   |Command _  -> Hashtbl.find table_command_key op
-   |Edit _     -> Hashtbl.find table_edit_key op
-   |Browse _   -> Hashtbl.find table_browse_key op
-   |Abbrev _   -> Hashtbl.find table_abbrev_key op
-   |IntEdit _  -> Hashtbl.find table_intedit_key op
-   |VarEdit _  -> Hashtbl.find table_varedit_key op
+   |Function x -> Hashtbl.find table_function_key x
+   |Command x  -> Hashtbl.find table_command_key x
+   |Edit x     -> Hashtbl.find table_edit_key x
+   |Browse x   -> Hashtbl.find table_browse_key x
+   |Abbrev x   -> Hashtbl.find table_abbrev_key x
+   |IntEdit x  -> Hashtbl.find table_intedit_key x
+   |VarEdit x  -> Hashtbl.find table_varedit_key x
 
 
 (* abbreviations used in abbreviation entry mode *)
-let abbrev_commands = ref "";;
+let abbrev_commands = ref [];;
 let abbrev_command_table = Hashtbl.create 50;;
 let command_abbrev_table = Hashtbl.create 50;;
 
 (* Register an abbreviation for an operation
  * This updates the string used in regexp matching, and
- * updates the hashtable used to find the corresponding operation. *)
+ * updates the hashtable used to find the corresponding operation. 
+ * Note: this list is generated in reverse order, so that
+ * the list of matches can be generated rapidly in the opposite
+ * order. *)
 let register_abbrev abbr op =
    (* Dummyproofing: if an abbreviation is a prefix of another
     * abbreviation, then it *must* lie earlier in the search order.
     * If not, it becomes impossible to execute the prefix command. *)
-   let regex = Str.regexp ("^" ^ abbr ^ ".*$") in
-   (try
-      let match_pos = Str.search_forward regex !abbrev_commands 0 in
-      let before = Str.string_before !abbrev_commands match_pos
-      and after  = Str.string_after !abbrev_commands match_pos in
-      abbrev_commands := before ^ abbr ^ "\n" ^ after
-   with Not_found ->
-      abbrev_commands := !abbrev_commands ^ abbr ^ "\n");
+   let regex = Str.regexp_string abbr in
+   let check_match (prev_result : bool) el =
+      if prev_result then
+         true
+      else
+         Str.string_match regex el 0
+   in
+   if List.fold_left check_match false !abbrev_commands then
+      (* if abbr is a prefix of some element, then it must
+       * go at the end of the list *)
+      abbrev_commands := !abbrev_commands @ [abbr]
+   else
+      (* if it has no prefix, then prepend it *)
+      abbrev_commands := abbr :: !abbrev_commands;
    Hashtbl.add abbrev_command_table abbr op;
    Hashtbl.add command_abbrev_table op abbr;;
    
+
+
+(* tables used in for constant entry *)
+let constant_symbols = ref [];;
+let constants_table = Hashtbl.create 50;;
+
+
+(* Register a constant string.
+ * This updates the string used in regexp matching, and
+ * adds the constant definition to a lookup table.
+ * Note: this list is generated in reverse order, so that
+ * the list of matches can be generated rapidly in the opposite
+ * order. *)
+let register_constant (const : string) (unit_def : Units.unit_def_t) =
+   (* Dummyproofing: if a constant is a prefix of another
+    * constant, then it *must* lie earlier in the search order.
+    * If not, it becomes impossible to execute the prefix command. *)
+   let regex = Str.regexp_string const in
+   let check_match (prev_result : bool) el =
+      if prev_result then
+         true
+      else
+         Str.string_match regex el 0
+   in
+   if List.fold_left check_match false !constant_symbols then
+      (* if const is a prefix of some element, then it must
+       * go at the end of the list *)
+      constant_symbols := !constant_symbols @ [const]
+   else
+      (* if it has no prefix, then prepend it *)
+      constant_symbols := const :: !constant_symbols;
+   Hashtbl.add constants_table const unit_def;;
+
+
 (* remove an abbreviation for a command. *)
 let unregister_abbrev abbr =
-   let regex = Str.regexp ("^" ^ abbr ^ "$") in
-   abbrev_commands := Str.replace_first regex "" !abbrev_commands;
+   let remove_matching out_list el =
+      if el = abbr then out_list
+      else el :: out_list
+   in
+   let sublist = 
+      List.fold_left remove_matching [] !abbrev_commands 
+   in
+   abbrev_commands := List.rev sublist;
    try
       let op = Hashtbl.find abbrev_command_table abbr in
       Hashtbl.remove abbrev_command_table abbr;
@@ -145,7 +199,8 @@ let translate_abbrev abb =
 let abbrev_of_operation op =
    Hashtbl.find command_abbrev_table op;;
 
-
+let translate_constant const =
+   Hashtbl.find constants_table const;;
 
 
 let decode_single_key_string key_string =
@@ -234,7 +289,7 @@ let decode_single_key_string key_string =
       else 
          let octal_regex = Str.regexp "^0o" in
          try
-            let pos = Str.search_forward octal_regex key_string 0 in
+            let _ = Str.search_forward octal_regex key_string 0 in
             ((int_of_string key_string), ("\\" ^ Str.string_after key_string
             2))
          with
@@ -248,27 +303,27 @@ let decode_single_key_string key_string =
  * between curses chtypes and commands (in both directions). *)
 let register_binding_internal k k_string op =
    match op with
-   |Function _ ->
-      Hashtbl.add table_key_function k op;
-      Hashtbl.add table_function_key op k_string
-   |Command _ ->
-      Hashtbl.add table_key_command k op;
-      Hashtbl.add table_command_key op k_string
-   |Edit _ ->
-      Hashtbl.add table_key_edit k op;
-      Hashtbl.add table_edit_key op k_string
-   |Browse _ ->
-      Hashtbl.add table_key_browse k op;
-      Hashtbl.add table_browse_key op k_string
-   |Abbrev _ ->
-      Hashtbl.add table_key_abbrev k op;
-      Hashtbl.add table_abbrev_key op k_string
-   |IntEdit _ ->
-      Hashtbl.add table_key_intedit k op;
-      Hashtbl.add table_intedit_key op k_string
-   |VarEdit _ ->
-      Hashtbl.add table_key_varedit k op;
-      Hashtbl.add table_varedit_key op k_string
+   |Function x ->
+      Hashtbl.add table_key_function k x;
+      Hashtbl.add table_function_key x k_string
+   |Command x ->
+      Hashtbl.add table_key_command k x;
+      Hashtbl.add table_command_key x k_string
+   |Edit x ->
+      Hashtbl.add table_key_edit k x;
+      Hashtbl.add table_edit_key x k_string
+   |Browse x ->
+      Hashtbl.add table_key_browse k x;
+      Hashtbl.add table_browse_key x k_string
+   |Abbrev x ->
+      Hashtbl.add table_key_abbrev k x;
+      Hashtbl.add table_abbrev_key x k_string
+   |IntEdit x ->
+      Hashtbl.add table_key_intedit k x;
+      Hashtbl.add table_intedit_key x k_string
+   |VarEdit x ->
+      Hashtbl.add table_key_varedit k x;
+      Hashtbl.add table_varedit_key x k_string
 
 
 (* convenience routine for previous *)
@@ -342,27 +397,27 @@ let unregister_varedit_binding key_string =
 (* Remove a key binding. *)
 let remove_binding k op =
    match op with
-   |Function _ ->
+   |Function x ->
       Hashtbl.remove table_key_function k;
-      Hashtbl.remove table_function_key op
-   |Command _ ->
+      Hashtbl.remove table_function_key x
+   |Command x ->
       Hashtbl.remove table_key_command k;
-      Hashtbl.remove table_command_key op
-   |Edit _ ->
+      Hashtbl.remove table_command_key x
+   |Edit x ->
       Hashtbl.remove table_key_edit k;
-      Hashtbl.remove table_edit_key op
-   |Browse _ ->
+      Hashtbl.remove table_edit_key x
+   |Browse x ->
       Hashtbl.remove table_key_browse k;
-      Hashtbl.remove table_browse_key op
-   |Abbrev _ ->
+      Hashtbl.remove table_browse_key x
+   |Abbrev x ->
       Hashtbl.remove table_key_abbrev k;
-      Hashtbl.remove table_abbrev_key op
-   |IntEdit _ ->
+      Hashtbl.remove table_abbrev_key x
+   |IntEdit x ->
       Hashtbl.remove table_key_intedit k;
-      Hashtbl.remove table_intedit_key op
-   |VarEdit _ ->
+      Hashtbl.remove table_intedit_key x
+   |VarEdit x ->
       Hashtbl.remove table_key_varedit k;
-      Hashtbl.remove table_varedit_key op
+      Hashtbl.remove table_varedit_key x
 
 
 (* Register a macro.  This parses the macro string and divides it into multiple
@@ -463,6 +518,7 @@ let operation_of_string command_str =
    |"command_undo"                  -> (Command Undo)
    |"command_begin_browsing"        -> (Command BeginBrowse)
    |"command_begin_abbrev"          -> (Command BeginAbbrev)
+   |"command_begin_constant"        -> (Command BeginConst)
    |"command_begin_variable"        -> (Command BeginVar)
    |"command_quit"                  -> (Command Quit)
    |"command_rad"                   -> (Command SetRadians)
@@ -497,14 +553,14 @@ let operation_of_string command_str =
    |"browse_keep"                   -> (Browse Keep)
    |"browse_keepn"                  -> (Browse KeepN)
    |"browse_edit"                   -> (Browse EditEntry)
-   |"abbrev_exit"                   -> (Abbrev ExitAbbrev)
-   |"abbrev_enter"                  -> (Abbrev EnterAbbrev)
+   |"abbrev_exit"                   -> (Abbrev AbbrevExit)
+   |"abbrev_enter"                  -> (Abbrev AbbrevEnter)
    |"abbrev_backspace"              -> (Abbrev AbbrevBackspace)
-   |"integer_cancel"                -> (IntEdit ExitIntEdit)
-   |"variable_cancel"               -> (VarEdit ExitVarEdit)
-   |"variable_enter"                -> (VarEdit EnterVarEdit)
+   |"integer_cancel"                -> (IntEdit IntEditExit)
+   |"variable_cancel"               -> (VarEdit VarEditExit)
+   |"variable_enter"                -> (VarEdit VarEditEnter)
    |"variable_backspace"            -> (VarEdit VarEditBackspace)
-   |"variable_complete"             -> (VarEdit CompleteVarEdit)
+   |"variable_complete"             -> (VarEdit VarEditComplete)
    |"function_rand"                 -> config_failwith 
                                        "operation \"function_rand\" is deprecated; please replace with \"command_rand\"."
    |"command_begin_extended"        -> config_failwith
@@ -740,6 +796,46 @@ let parse_line line_stream =
       | [< >] ->
          config_failwith ("Unmatched variable name after \"set\"")
       end
+   | [< 'Kwd "base_unit" >] ->
+      begin match line_stream with parser
+      | [< 'String base_u; 'String prefix_s >] ->
+         begin try
+            let prefix = Units.prefix_of_string prefix_s in
+            unit_table := Units.add_base_unit base_u prefix !unit_table
+         with Not_found ->
+            config_failwith 
+            ("Expected an SI prefix string (or null string) after: base_unit \"" ^
+            base_u ^ "\"")
+         end
+      | [< >] ->
+         config_failwith ("Expected a unit string and prefix string after \"base_unit\"")
+      end
+   | [< 'Kwd "unit" >] ->
+      begin match line_stream with parser
+      | [< 'String unit_str; 'String unit_def_str >] ->
+         begin try
+            let unit_def = Units.unit_def_of_string unit_def_str !unit_table in
+            unit_table := Units.add_unit unit_str unit_def !unit_table
+         with Units.Units_error s ->
+            config_failwith ("Illegal unit definition: unit \"" ^
+            unit_str ^ "\" \"" ^ unit_def_str ^ "\"; " ^ s)
+         end
+      | [< >] ->
+         config_failwith ("Expected a unit string and definition after \"unit\"")
+      end
+   | [< 'Kwd "constant" >] ->
+      begin match line_stream with parser
+      | [< 'String const_str; 'String unit_def_str >] ->
+         begin try
+            let unit_def = Units.unit_def_of_string unit_def_str !unit_table in
+            register_constant const_str unit_def
+         with Units.Units_error s ->
+            config_failwith ("Illegal constant definition: constant \"" ^
+            const_str ^ "\" \"" ^ unit_def_str ^ "\"; " ^ s)
+         end
+      | [< >] ->
+         config_failwith ("Expected a constant name and definition after \"constant\"")
+      end
    | [< 'Kwd "#" >] ->
       ()
    | [< >] ->
@@ -818,18 +914,20 @@ let open_rcfile rcfile_op =
          let homedir = Sys.getenv "HOME" in
          homedir ^ "/.orpierc"
       in
-      let prefix_rcfile = 
-         if Install.prefix = "/usr" || Install.prefix = "/usr/" then
-            "/etc/orpierc"
-         else
-            Install.prefix ^ "/etc/orpierc" 
+      let rcfile_fullpath = 
+         (* expand out any occurrences of ${prefix} that autoconf
+          * decides to insert *)
+         let prefix_regex = Str.regexp "\\${prefix}" in
+         let expanded_sysconfdir = Str.global_replace prefix_regex 
+         Install.prefix Install.sysconfdir in
+         Utility.join_path expanded_sysconfdir "orpierc"
       in
       begin try (open_in home_rcfile, home_rcfile)
       with Sys_error error_str ->
-         begin try (open_in prefix_rcfile, prefix_rcfile)
+         begin try (open_in rcfile_fullpath, rcfile_fullpath)
          with Sys_error error_str -> failwith 
             ("Could not open configuration file \"" ^ home_rcfile ^ "\" or \"" ^ 
-            prefix_rcfile ^ "\" .")
+            rcfile_fullpath ^ "\" .")
          end
       end
    |Some file ->
@@ -845,7 +943,7 @@ let rec process_rcfile rcfile_op =
          ["include"; "bind"; "unbind_function"; "unbind_command";
          "unbind_edit"; "unbind_browse"; "unbind_abbrev"; "unbind_integer";
          "unbind_variable"; "autobind"; "abbrev"; "unabbrev"; "macro"; "set"; 
-         "#"] 
+         "base_unit"; "unit"; "constant"; "#"] 
       (Stream.of_string line)
    in
    let empty_regexp = Str.regexp "^[\t ]*$" in
