@@ -1,10 +1,9 @@
-(*  Orpie -- a stack-based RPN calculator for the console
- *  Copyright (C) 2003-2004  Paul Pelzl
+(*  Orpie -- a fullscreen RPN calculator for the console
+ *  Copyright (C) 2003-2004, 2005, 2006-2007 Paul Pelzl
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  it under the terms of the GNU General Public License, Version 2,
+ *  as published by the Free Software Foundation.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -31,7 +30,13 @@
  * line-oriented and fullscreen displays, four different bases, two
  * different angle modes, and two different complex representations.  The
  * string representations are created immediately when needed; if not
- * immediately needed, they will eventually be filled in by a background thread.
+ * immediately needed, they will eventually be precomputed and filled in in by a 
+ * background thread (unless conserve_memory = false).
+ *
+ * Note: arguably, the rendering functions belong with the interface code
+ *       rather than with the stack code.  I may make this change at some
+ *       point, to better abstract the interface away from the underlying
+ *       calculator object.
  *)
 
 
@@ -44,11 +49,11 @@ exception Stack_error of string
 (* orpie_data_t values are returned by pop(), but the values
  * on the stack are in stack_data_t format *)
 type orpie_data_t = | RpcInt of Big_int.big_int
-                    | RpcFloatUnit of Units.unit_t
-                    | RpcComplexUnit of Units.unit_t
-                    | RpcFloatMatrixUnit of Gsl_matrix.matrix * Units.unit_t
+                    | RpcFloatUnit of float * Units.unit_set_t
+                    | RpcComplexUnit of Complex.t * Units.unit_set_t
+                    | RpcFloatMatrixUnit of Gsl_matrix.matrix * Units.unit_set_t
                     | RpcComplexMatrixUnit of Gsl_matrix_complex.matrix *
-                                              Units.unit_t
+                                              Units.unit_set_t
                     | RpcVariable of string
 
 type stack_int_string_t   = {mutable i_bin_line : string option; 
@@ -76,12 +81,12 @@ type stack_var_string_t   = {mutable v_line : string option;
 
 (* internal storage format of stack elements *)
 type stack_data_t = | StackInt of Big_int.big_int * stack_int_string_t
-                    | StackFloatUnit of Units.unit_t * stack_float_unit_string_t
-                    | StackComplexUnit of Units.unit_t * stack_cmpx_unit_string_t
-                    | StackFloatMatrixUnit of Gsl_matrix.matrix * Units.unit_t *
+                    | StackFloatUnit of float * Units.unit_set_t * stack_float_unit_string_t
+                    | StackComplexUnit of Complex.t * Units.unit_set_t * stack_cmpx_unit_string_t
+                    | StackFloatMatrixUnit of Gsl_matrix.matrix * Units.unit_set_t *
                                               stack_fmat_unit_string_t
                     | StackComplexMatrixUnit of Gsl_matrix_complex.matrix *
-                                                Units.unit_t * stack_cmat_string_t
+                                                Units.unit_set_t * stack_cmat_string_t
                     | StackVariable of string * stack_var_string_t
 
 let raise_invalid s = raise (Invalid_argument s)
@@ -89,8 +94,8 @@ let raise_invalid s = raise (Invalid_argument s)
 let orpie_data_of_stack_data (sd : stack_data_t) =
    match sd with
    |StackInt (ii, _)                   -> RpcInt ii
-   |StackFloatUnit (fu, _)             -> RpcFloatUnit fu
-   |StackComplexUnit (cc, _)           -> RpcComplexUnit cc
+   |StackFloatUnit (ff, uu, _)         -> RpcFloatUnit (ff, uu)
+   |StackComplexUnit (cc, uu, _)       -> RpcComplexUnit (cc, uu)
    |StackFloatMatrixUnit (fm, uu, _)   -> RpcFloatMatrixUnit (fm, uu)
    |StackComplexMatrixUnit (cm, uu, _) -> RpcComplexMatrixUnit (cm, uu)
    |StackVariable (vv, _)              -> RpcVariable vv
@@ -107,11 +112,11 @@ let stack_data_of_orpie_data (od : orpie_data_t) =
           i_oct_fs   = None;
           i_dec_fs   = None;
           i_hex_fs   = None})
-   |RpcFloatUnit fu ->
-      StackFloatUnit (fu,
+   |RpcFloatUnit (ff, uu) ->
+      StackFloatUnit (ff, uu,
          {fu = None})
-   |RpcComplexUnit cc ->
-      StackComplexUnit (cc, 
+   |RpcComplexUnit (cc, uu) ->
+      StackComplexUnit (cc, uu,
          {c_rect    = None;
           c_pol_rad = None;
           c_pol_deg = None})
@@ -133,26 +138,15 @@ let stack_data_of_orpie_data (od : orpie_data_t) =
           v_fs   = None})
 
 
-let funit_of_float ff = {
-   Units.coeff = {
-      Complex.re = ff;
-      Complex.im = 0.0
-   };
-   Units.factors = []
-}
+let funit_of_float (ff : float) : float * Units.unit_set_t = (ff, Units.empty_unit)
 
-let cunit_of_cpx cc = {
-   Units.coeff   = cc;
-   Units.factors = []
-}
+let cunit_of_cpx cc = (cc, Units.empty_unit)
 
-let unorm uu = {
-   Units.coeff   = Complex.one;
-   Units.factors = uu.Units.factors
-}
+let unorm uu = (1.0, uu)
 
-let has_units fu =
-   fu.Units.factors <> []
+let c_of_f ff = {Complex.re = ff; Complex.im = 0.0}
+
+let has_units uu = uu <> Units.empty_unit
 
 type display_mode_t = | Line | Fullscreen
 
@@ -166,89 +160,29 @@ type calculator_modes = {angle : angle_mode; base : base_mode;
 let size_inc = 100
 let pi = 3.14159265358979323846
 
-class rpc_stack =
+class rpc_stack conserve_memory_in =
    object(self)
       val mutable len = 0
-      val mutable stack = Array.make size_inc (stack_data_of_orpie_data
-      (RpcFloatUnit (funit_of_float 0.0)))
+      val mutable stack = 
+         let (f0, u0) = funit_of_float 0.0 in
+         Array.make size_inc (stack_data_of_orpie_data
+         (RpcFloatUnit (f0, u0)))
+      val conserve_memory = conserve_memory_in
       val render_stack = Stack.create ()
 
 
       method length = len
 
 
-      (* save to a datafile using the Marshal module *)
-      method save_state (modes : calculator_modes) 
-      (variables : (string, orpie_data_t) Hashtbl.t) =
-         try
-            let version_file = Utility.join_path !(Rcfile.datadir) "version" in
-            let version_channel = Utility.open_or_create_out_bin version_file in
-            output_string version_channel Version.version;
-            close_out version_channel;
-            let save_file = Utility.join_path !(Rcfile.datadir) "calc_state" in
-            let save_channel = Utility.open_or_create_out_bin save_file in
-            Marshal.to_channel save_channel 
-            (modes, variables, !Rcfile.autobind_keys, stack, len) [];
-            close_out save_channel
-         with
-            |Sys_error ss -> raise (Invalid_argument "can't open data file for writing")
-            |Failure ff   -> raise (Invalid_argument "can't serialize calculator data to file")
+      method get_state () = (stack, len)
 
-      (* load from a datafile using the Marshal module *)
-      (* FIXME: if the datafile is corrupted, this can segfault... *)
-      method load_state () =
-         try
-            (* check whether the version file exists *)
-            let version_file = Utility.join_path !(Rcfile.datadir) "version" in
-            if Sys.file_exists (Utility.expand_file version_file) then begin
-               (* if it does exist, try loading it *)
-               let version_channel = 
-                  Utility.expand_open_in_ascii version_file
-               in
-               let ver_string = input_line version_channel in
-               close_in version_channel;
-               (* if the version strings match, then assume it's okay to use
-                * Marshal. *)
-               if ver_string = Version.version then begin
-                  (* check whether the state file exists *)
-                  let datafile = Utility.join_path !(Rcfile.datadir) "calc_state" in
-                  if Sys.file_exists (Utility.expand_file datafile) then begin
-                     (* if it does exist, try loading it *)
-                     let load_channel = Utility.expand_open_in_bin datafile in
-                     let data_modes, data_variables, data_autobind_keys, data_stack, data_len = 
-                        (Marshal.from_channel load_channel : calculator_modes * 
-                        ((string, orpie_data_t) Hashtbl.t) * 
-                        (int * string * Operations.operation_t option * int) array *
-                        (stack_data_t array) * int)
-                     in
-                     close_in load_channel;
-                     stack <- data_stack;
-                     len <- data_len;
-                     Rcfile.validate_saved_autobindings data_autobind_keys;
-                     data_modes, data_variables
-                  end else
-                     (* if the datafile is missing, do nothing as it will be
-                      * created later *)
-                     ({angle = Rad; base = Dec; complex = Rect}, Hashtbl.create 20)
-               end else
-                  (* if the version strings don't match, don't try loading anything *)
-                  ({angle = Rad; base = Dec; complex = Rect}, Hashtbl.create 20)
-            end else
-               (* if the version file does not exist, don't try loading anything *)
-                  ({angle = Rad; base = Dec; complex = Rect}, Hashtbl.create 20)
-         with
-            (* this gets raised if, for example, we don't have read permission
-             * on the state data file *)
-            |Sys_error ss -> raise (Invalid_argument "can't open calculator state data file")
-            (* this shouldn't happen unless the data file gets corrupted. *)
-            |Failure ff -> raise (Invalid_argument "can't deserialize calculator data from file")
+      method set_state (s, l) =
+         stack <- s;
+         len   <- l
 
-            
       method backup () =
-         let b_stack = Array.copy stack 
-         and b_len = len in
-         {< len = b_len; stack = b_stack >}
-
+         let b_stack = Array.copy stack in
+         {< stack = b_stack >}
 
 
       method private expand_size () =
@@ -256,7 +190,7 @@ class rpc_stack =
          if len >= Array.length stack then begin
             let new_stack = Array.make ((Array.length stack) + size_inc)
             (stack_data_of_orpie_data (RpcFloatUnit 
-            (Units.unit_of_float_string 0.0 ""))) in
+            (0.0, Units.empty_unit))) in
             Array.blit stack 0 new_stack 0 (Array.length stack);
             stack <- new_stack
          end else
@@ -268,7 +202,7 @@ class rpc_stack =
          let new_el = stack_data_of_orpie_data v in
          stack.(len) <- new_el;
          len <- len + 1;
-         if !Rcfile.conserve_memory then ()
+         if conserve_memory then ()
          else Stack.push new_el render_stack
 
 
@@ -464,18 +398,18 @@ class rpc_stack =
                   |Hex -> lookup_int_str ii_str.i_hex_fs
                   end
                end
-            |StackFloatUnit (fu, fu_str) ->
+            |StackFloatUnit (ff, uu, fu_str) ->
                begin match fu_str.fu with
                |None ->
-                  self#create_float_unit_string fu fu_str
+                  self#create_float_unit_string ff uu fu_str
                |Some ss ->
                   ss
                end
-            |StackComplexUnit (cc, cc_str) ->
+            |StackComplexUnit (cc, uu, cc_str) ->
                let lookup_cmpx_str record =
                   begin match record with
                   |None ->
-                     self#create_cmpx_unit_string calc_modes cc cc_str
+                     self#create_cmpx_unit_string calc_modes cc uu cc_str
                   |Some ss ->
                      ss
                   end
@@ -581,18 +515,18 @@ class rpc_stack =
                ii_str.i_hex_line
             (* the remaining integer strings will get filled in as
              * side-effects of the previous *)
-         |StackFloatUnit (fu, fu_str) ->
+         |StackFloatUnit (ff, uu, fu_str) ->
             begin match fu_str.fu with
             |None ->
-               let _ = self#create_float_unit_string fu fu_str in ()
+               let _ = self#create_float_unit_string ff uu fu_str in ()
             |Some ss ->
                ()
             end
-         |StackComplexUnit (cc, cc_str) ->
+         |StackComplexUnit (cc, uu, cc_str) ->
             let lookup_cmpx_str c_mode record =
                begin match record with
                |None ->
-                  let _ = self#create_cmpx_unit_string c_mode cc cc_str in ()
+                  let _ = self#create_cmpx_unit_string c_mode cc uu cc_str in ()
                |Some ss ->
                   ()
                end
@@ -658,7 +592,7 @@ class rpc_stack =
             let s = string_of_big_int_base ii 2 in
             let line = "# " ^ s ^ "`b"
             and fs   = "#" ^ s ^ "`b" in
-            if !Rcfile.conserve_memory then () else begin
+            if conserve_memory then () else begin
                ii_str.i_bin_line <- Some line;
                ii_str.i_bin_fs   <- Some fs
             end;
@@ -670,7 +604,7 @@ class rpc_stack =
             let s = string_of_big_int_base ii 8 in
             let line = "# " ^ s ^ "`o"
             and fs   = "#" ^ s ^ "`o" in
-            if !Rcfile.conserve_memory then () else begin
+            if conserve_memory then () else begin
                ii_str.i_oct_line <- Some line;
                ii_str.i_oct_fs   <- Some fs
             end;
@@ -682,7 +616,7 @@ class rpc_stack =
             let s = string_of_big_int_base ii 16 in
             let line = "# " ^ s ^ "`h"
             and fs   = "#" ^ s ^ "`h" in
-            if !Rcfile.conserve_memory then () else begin
+            if conserve_memory then () else begin
                ii_str.i_hex_line <- Some line;
                ii_str.i_hex_fs   <- Some fs
             end;
@@ -694,7 +628,7 @@ class rpc_stack =
             let s = string_of_big_int_base_gen ii 10 in
             let line = "# " ^ s ^ "`d"
             and fs   = "#" ^ s ^ "`d" in
-            if !Rcfile.conserve_memory then () else begin
+            if conserve_memory then () else begin
                ii_str.i_dec_line <- Some line;
                ii_str.i_dec_fs   <- Some fs
             end;
@@ -705,51 +639,49 @@ class rpc_stack =
 
 
       (* generate a string representation for a floating-point value with a unit *)
-      method private create_float_unit_string fu fu_str =
+      method private create_float_unit_string ff uu fu_str =
          let s = 
-            if fu.Units.factors <> [] then
-               sprintf "%.15g_%s" fu.Units.coeff.Complex.re 
-               (Units.string_of_unit fu.Units.factors)
+            if uu <> Units.empty_unit then
+               sprintf "%.15g_%s" ff
+               (Units.string_of_units uu)
             else
-               sprintf "%.15g" fu.Units.coeff.Complex.re 
+               sprintf "%.15g" ff
          in
-         if !Rcfile.conserve_memory then () 
+         if conserve_memory then () 
          else fu_str.fu <- Some s;
          s
 
 
       (* generate a string representation for a complex value, taking
        * into account the representation mode and angle mode of the calc *)
-      method private create_cmpx_unit_string calc_modes cc cc_str =
+      method private create_cmpx_unit_string calc_modes cc uu cc_str =
          let append_units ss =
-            if cc.Units.factors <> [] then
-               ss ^ "_" ^ (Units.string_of_unit cc.Units.factors)
+            if uu <> Units.empty_unit then
+               ss ^ "_" ^ (Units.string_of_units uu)
             else
                ss
          in
          match calc_modes.complex with
          |Rect ->
             let s = append_units 
-            (sprintf "(%.15g, %.15g)" cc.Units.coeff.Complex.re 
-            cc.Units.coeff.Complex.im) in
-            if !Rcfile.conserve_memory then () 
+            (sprintf "(%.15g, %.15g)" cc.Complex.re cc.Complex.im) in
+            if conserve_memory then () 
             else cc_str.c_rect <- Some s;
             s
          |Polar ->
-            let r = sqrt (cc.Units.coeff.Complex.re *. cc.Units.coeff.Complex.re +.
-            cc.Units.coeff.Complex.im *. cc.Units.coeff.Complex.im)
-            and theta = atan2 cc.Units.coeff.Complex.im 
-            cc.Units.coeff.Complex.re in
+            let r = sqrt (cc.Complex.re *. cc.Complex.re +.
+               cc.Complex.im *. cc.Complex.im)
+            and theta = atan2 cc.Complex.im cc.Complex.re in
             begin match calc_modes.angle with
             |Rad ->
                let s = append_units (sprintf "(%.15g <%.15g)" r theta) in
-               if !Rcfile.conserve_memory then () 
+               if conserve_memory then () 
                else cc_str.c_pol_rad <- Some s;
                s
             |Deg ->
                let s = append_units 
                (sprintf "(%.15g <%.15g)" r (180.0 /. pi *. theta)) in
-               if !Rcfile.conserve_memory then () 
+               if conserve_memory then () 
                else cc_str.c_pol_deg <- Some s;
                s
             end
@@ -760,7 +692,7 @@ class rpc_stack =
       method private create_fmat_unit_string disp_mode fm uu fm_str =
          let append_units ss = 
             if has_units uu then
-               ss ^ "_" ^ Units.string_of_unit uu.Units.factors
+               ss ^ "_" ^ Units.string_of_units uu
             else
                ss
          in
@@ -782,7 +714,7 @@ class rpc_stack =
                !line
             in
             let ss = append_units s in
-            if !Rcfile.conserve_memory then () 
+            if conserve_memory then () 
             else fm_str.fmat_line <- Some ss;
             ss
          |Fullscreen ->
@@ -822,7 +754,7 @@ class rpc_stack =
                !line
             in
             let ss = append_units s in
-            if !Rcfile.conserve_memory then () 
+            if conserve_memory then () 
             else fm_str.fmat_fs <- Some ss;
             ss
 
@@ -832,8 +764,8 @@ class rpc_stack =
        * taking into account the display mode. *)
       method private create_cmat_unit_string disp_mode calc_modes cm uu cm_str =
          let append_units ss = 
-            if has_units uu then
-               ss ^ "_" ^ Units.string_of_unit uu.Units.factors
+            if uu <> Units.empty_unit then
+               ss ^ "_" ^ Units.string_of_units uu
             else
                ss
          in
@@ -888,7 +820,7 @@ class rpc_stack =
                !line
             in
             let ss = append_units s in
-            if !Rcfile.conserve_memory then 
+            if conserve_memory then 
                () 
             else
                begin match calc_modes.complex with
@@ -1006,7 +938,7 @@ class rpc_stack =
                !line
             in
             let ss = append_units s in
-            if !Rcfile.conserve_memory then 
+            if conserve_memory then 
                () 
             else
                begin match calc_modes.complex with
@@ -1027,7 +959,7 @@ class rpc_stack =
       method private create_var_string disp_mode vv vv_str =
          let line = "@ " ^ vv
          and fs   = "@" ^ vv in
-         if !Rcfile.conserve_memory then 
+         if conserve_memory then 
             () 
          else begin
             vv_str.v_line <- Some line;
